==> cosmo_1.0.tar.gz: cosmo/src/back.c <==
/*****************************************************************************
*			     					             *
*      cosmo - Constrained search for motifs in DNA sequences                *
*      Copyright (C) 2006  Oliver Bembom (bembom@berkeley.edu)               *
*                                                                            *
*      This file is part of cosmo.                                           *
*                                                                            *
*      cosmo is free software; you can redistribute it and/or                *
*      modify it under the terms of the GNU Lesser General Public            *
*      License as published by the Free Software Foundation; either          *
*      version 2.1 of the License, or (at your option) any later version.    *
*                                                                            *
*      cosmo is distributed in the hope that it will be useful,              *
*      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
*      Lesser General Public License for more details.                       *
*                                                                            *
*      You should have received a copy of the GNU Lesser General Public      *
*      License along with this program; if not, write to the Free Software   *
*      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,            *
*      MA  02110-1301, USA.                                                  *
*                                                                            *
*                                                                            *
*****************************************************************************/
  

/*****************************************************************************
*                                                                            *
*      This module contains functions for estimating the transition matrix   *
*      of the background Markov model.                                       *
*                                                                            *
*****************************************************************************/

#include <R_ext/PrtUtil.h>
#include <R_ext/Memory.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <float.h>
#include "macros.h"
#include "cosmo.h"
#include "back.h"
#include "fit.h"
#include "matLib.h"
#include "precalc.h"
   

/**********************************************************************/
/*
  getNumRows

  Get the number of rows in a transition matrix of order 'k' given by

  sum_{j=0}^k 4^j

*/
/**********************************************************************/
long getNumRows(
		int order  /* order for which to get number of rows */
		)
{
  int i;             /* index */
  long numRows;      /* number of rows */


  numRows = 1;
  for (i=0; i<order;i++){
    numRows += pow(4,i+1);
  }

  if (DEBUG>=2) Rprintf("order = %d and numRows = %ld\n", order, numRows);
  return numRows;
}


/**********************************************************************/
/*
  getTransMat

  Get transition matrix for background Markov model.
  Order may be chosen by likelihood-based cross-validation.
  Memory is allocated for transition matrices and matrices are stored
  in dataset->transMats.
  

*/
/**********************************************************************/
int getTransMat(
		 BOOLEAN cv,           /* find order of transition matrix by CV? */
		 DATASET *dataSet      /* the dataset */
		 )
{
  
  int subOrder;   /* orders from 0 up to the selected order for edge of sequences */ 
  int order;      /* selected order of Markov model */
  long numRows;   /* number of rows in a given transition matrix */

  /* allocate space for transition matrices */
  dataSet->transMats = p2DoubleMalloc(dataSet->maxOrder + 1);
  for (order=0; order<=dataSet->maxOrder; order++){
    numRows = pow(4,order);
    dataSet->transMats[order] = DoubleMalloc(4*numRows);
  }

  
  /* if desired, choose order by cross-validation */
  if (cv) dataSet->order = cvOrder(dataSet);
  if(DEBUG>=3) Rprintf("getTransMat: Estimating transition matrix for order = %d\n\n", dataSet->order);
  
  /* for each order from 0 up to chosen order, estimate the transition matrix */
  for (subOrder=0; subOrder<=dataSet->order; subOrder++){
    if (!estTransMat(subOrder, FALSE, 0, dataSet)){
      Rprintf("getTransMat: Can't estimate transition matrix of order %d: Zero cells. Exiting ...\n", subOrder);
      return(0);
    }
  }

  if (DEBUG>=4){
    for (subOrder=0; subOrder<=dataSet->order; subOrder++){
      Rprintf("order = %d\n", subOrder);
      PrintDoubleMatrix2File(stderr,dataSet->transMats[subOrder], 4, pow(4,subOrder));
      Rprintf("\n");
    }
  }

  return(1);
}


/**********************************************************************/
/*
  cvOrder  
  
  Choose order of background Markov model by likelihood-based CV.

*/
/**********************************************************************/
int cvOrder(
	    DATASET *dataSet   /* the dataset */
	    )
{
  int order;         /* candidate order */
  int maxOrder;      /* maximum order to consider */
  int subOrder;      /* lower orders */
  int valGroup;      /* validation group */
  int fold;          /* fold for CV */
  double KL;         /* Kullback-Leibler divergence */
  double minKL;      /* minimum Kullback-Leibler divergence */
  BOOLEAN goodLik;   /* is this likelihood valid, i.e. not equal to minus infinity? */
  int bestOrder;     /* best order so far */
  int numSeqs;       /* number of sequences */
  SAMPLE *samples;   /* the samples */
  BOOLEAN thisOne;
  BOOLEAN goodEst;

  maxOrder = dataSet->maxOrder;
  goodLik  = TRUE;
  goodEst  = TRUE;
  samples  = dataSet->hasBackFile ? dataSet->backSamples : dataSet->samples;
  numSeqs  = dataSet->hasBackFile ? dataSet->numBackSeqs : dataSet->numSeqs;
  fold     = dataSet->backFold;

  /* divide samples into validation groups */
  createCvGroups(samples, numSeqs, fold);
  
  bestOrder = 0;
  minKL     = BIG;
  for (order=0; order<maxOrder;order++) dataSet->backKL[order] = 0;
  
  for (order=0; order<=maxOrder && goodEst==TRUE; order++){

    KL = 0;
    for (valGroup=0; valGroup<fold; valGroup++){

      /* estimate all the transition matrices up to this order from learning set */
      for (subOrder=0; subOrder<=order; subOrder++)goodEst = goodEst && estTransMat(subOrder, TRUE, valGroup, dataSet); 

      KL += getMarkovKL(order, &goodLik, TRUE, valGroup, dataSet);
      goodEst = goodEst & goodLik;
    }
    KL /= fold;

    KL = goodEst ? KL : BIG;
    dataSet->backKL[order] = KL;

    if (DEBUG>=3) Rprintf("order = %d KL = %lf\n", order, KL);

    if (KL < minKL){
      minKL = KL;
      bestOrder = order;
    }
  }

  if (DEBUG>=2) Rprintf( "cvOrder: Order of background Markov model estimated as order = %d by CV\n", bestOrder);

  return bestOrder;

}


/**********************************************************************/
/*

  getMarkovKl
  
  Get Kullback-Leibler divergence for a given estimate of the transition
  matrix.

  Returns Kullback-Leibler divergence.

*/
/**********************************************************************/
double getMarkovKL(
		   int order,              /* order of transition matrix */
		   BOOLEAN *goodLik,       /* is the likelihood not equal to zero? */
		   BOOLEAN forCV,          /* evaluate Kullback-Leibler divergence on a subset of the sequences? */
		   int valGroup,           /* the validation group on which to evaluate KL divergence */
		   DATASET *dataSet        /* the dataset */
		   )
{

  int seqNum;            /* sequence number */
  int numSeqs;           /* number of sequences */
  int start;             /* position in sequence */
  double *transMat;      /* transition matrix */
  int *seq;              /* current sequence */
  long seqLength;        /* length of current sequence */
  long index;            /* index in transition matrix */
  long numRows;          /* number of rows in transition matrix */
  SAMPLE *sample;        /* current sample */
  SAMPLE *samples;       /* collection of samples */
  double logZero;        /* constant: approximate log(0) */
  double lik;            /* likelihood */ 
  int effNumSeqs;        /* effective number of sequences */

  logZero  = LOG(0);
  samples  = dataSet->hasBackFile ? dataSet->backSamples : dataSet->samples;
  numSeqs  = dataSet->hasBackFile ? dataSet->numBackSeqs : dataSet->numSeqs;
  numRows  = pow(4,order);
  *goodLik  = TRUE;

  lik = 0;
  effNumSeqs = 0;
  for (seqNum=0; seqNum<numSeqs; seqNum++){
    sample      = samples + seqNum;
    seq         = sample->seq;
    seqLength   = sample->length;

    if (forCV && sample->cvGroup!=valGroup) continue;

    effNumSeqs ++;

    for (start=0; start<order; start++){
      if (seq[start]<4){                         /* normal characters */
	transMat = dataSet->transMats[start];
	index    = findIndex(seq+start, start+1);
	if (transMat[index]==logZero) *goodLik = FALSE;
	lik    += transMat[index];
      }else{
	/* unknown character 'X' appears with probability 0.25 */
	lik    += LOG(1.0/4);
      }
    }

    transMat   = dataSet->transMats[order];
    for (start=order; start<seqLength-order; start++){
      if (seq[start]<4){                           /* normal characters */
	index    = findIndex(seq+start, order+1);
	if (transMat[index]==logZero) *goodLik = FALSE;
	lik     += transMat[index];
      }else{
	/* unknown character 'X' appears with probability 0.25 */
	lik     += LOG(1.0/4);
      }
    }
  }
    
  return -lik/effNumSeqs;
}


/**********************************************************************/
/*

  estTransMat
  
  Estimate transition matrix for a given order.
  This may be done on a subset of sequences by leaving out those 
  sequences in a specified validation group for CV purposes.

  Returns TRUE if the transition matrix could be estimated without
  zero cells and FALSE otherwise.

*/
/**********************************************************************/
BOOLEAN estTransMat(
		    int order,        /* order of the transition matrix */
		    BOOLEAN forCV,    /* are some subsequences to be left out? */
		    int valGroup,     /* validation group that is to be excluded  */
		    DATASET *dataSet  /* the dataset */
		    )
{
  int seqNum;            /* sequence number */
  int numSeqs;           /* number of sequences */
  int start;             /* position in sequence */
  double *transMat;      /* transition matrix */
  int *seq;              /* current sequence */
  long seqLength;        /* length of current sequence */
  long index;            /* index in transition matrix */
  long numRows;          /* number of rows in transition matrix */
  long rowNum;           /* current row number */
  SAMPLE *sample;        /* current sample */
  SAMPLE *samples;       /* collection of samples */
  int nuc;               /* nucleotide */
  double sum;            /* sum of counts for a given preceding oligo for normalization of counts to frequencies */   
  BOOLEAN goodTransMat;  /* does the transition matrix contain zero cells? */

  
  goodTransMat = TRUE;
  transMat = dataSet->transMats[order];
  samples  = dataSet->hasBackFile ? dataSet->backSamples : dataSet->samples;
  numSeqs  = dataSet->hasBackFile ? dataSet->numBackSeqs : dataSet->numSeqs;
  numRows  = pow(4,order);

  /* initialize transition matrix to 0 */
  for (rowNum=0; rowNum < numRows; rowNum++){
    for (nuc=0; nuc<4; nuc++){
      transMat[rowNum*4 + nuc] = 0;
    }
  }

  /* fill transition matrix with observed counts */
  for (seqNum=0; seqNum<numSeqs; seqNum++){
    sample      = samples + seqNum;
    seq         = sample->seq;
    seqLength   = sample->length;


    if (forCV && sample->cvGroup==valGroup) continue;

    for (start=0; start< seqLength-order; start++){

      /* ignore any tuples that contain X */
      if (!containsX(seq+start,order+1,4)){
	index = findIndex(seq+start, order+1);
	transMat[index] ++;
      }
    }
  }

  if (DEBUG>=3) PrintDoubleMatrix2File(stderr,transMat,4,numRows);

  /* convert frequencies to probabilities */
  for (rowNum=0; rowNum<numRows; rowNum++){
    sum = 0;

    for (nuc=0; nuc<4; nuc++) sum += transMat[4*rowNum + nuc];    
    for (nuc=0; nuc<4; nuc++){
      if (transMat[4*rowNum + nuc]==0) goodTransMat = FALSE;
      transMat[4*rowNum + nuc] = LOG(transMat[4*rowNum + nuc]/sum);
    }
  }

  if (DEBUG>=3) PrintDoubleMatrix2File(stderr,transMat,4,numRows);
  
  return goodTransMat;

}

/**********************************************************************/
/*

  containsX
  
  Returns TRUE if the tuple contains the unknown character 'X', coded
  in integer format as the alphabet length. Returns FALSE otherwise.

*/
/**********************************************************************/
BOOLEAN containsX(
		  int * tuple,      /* the tuple to test */
		  int length,       /* length of the tuple */
		  int alen          /* length of the alphabet */
		  )
{
     int i;

     for (i=0; i<length; i++){
       if (tuple[i]>=alen) return(TRUE);
     }

     return(FALSE);      
}

==> cosmo_1.6.0.tar.gz: cosmo/src/back.c <==
/******************************************************************************
*                                                                             *
*      cosmo - Constrained search for motifs in DNA sequences                 *
*      Author: Oliver Bembom                                                  *
*                                                                             *
*      This file is part of cosmo.                                            *
*                                                                             *
*      Conditions of use:                                                     *
*                                                                             *
*    1. cosmo is under the exclusive copyright of Oliver Bembom,              *
*       (bembom@berkeley.edu), the Regents of the University of California,   *
*       and Peter Spellucci (spellucci@mathematik.tu-darmstadt.de).           *
*                                                                             *
*    2. Permission to use, copy, modify, and distribute any part of this      *
*       software for research purposes, without fee and without a written     *
*       agreement, is hereby granted, provided that this license appears in   *
*       all copies. Commercial uses require permission and licensing from     *
*       Oliver Bembom, the Regents of the University of California, and       *
*       Peter Spellucci.                                                      *
*                                                                             *
*    3. If you use this software in your research, please cite                *
*                                                                             *
*       O. Bembom, S. Keles, M.J. van der Laan (2007). Supervised Detection   *
*       of Conserved Motifs in DNA Sequences with cosmo. Statistical          *
*       Applications in Genetics and Molecular Biology: Vol. 6 : Iss. 1,      *
*       Article 8. http://www.bepress.com/sagmb/vol6/iss1/art8.               *
*                                                                             *
*       cosmo makes use of the donlp2() function by Peter Spellucci.          *
*       The use of donlp2() must be acknowledged in any publication which     *
*       contains results obtained with cosmo or parts of it. Citation of the  *
*       author's name and netlib-source is suitable.                          *
*                                                                             *
*    4. In no event shall the author be liable to any party for direct, in-   *
*       direct, special, incidental, or consequential damages, including lost *
*       profits, arising out of the use of this software, even if the author  *
*       has been advised of the possibility of such damage.                   *
*                                                                             *
*    5. cosmo is provided on an 'as is' basis, and the author has no obli-    *
*       gations to provide maintenance, support, updates, enhancements, or    *
*       modifications. The author makes no representations and extends no     *
*       warranties of any kind, either expressed or implied, including, but   *
*       not limited to, the implied warranties of merchantability or fitness  *
*       for a particular purpose, or that the use of the material will not    *
*       infringe any patent, trademark or other rights.                       *
*                                                                             *
*                                                                             *
******************************************************************************/
  

/*****************************************************************************
*                                                                            *
*      This module contains functions for estimating the transition matrix   *
*      of the background Markov model.                                       *
*                                                                            *
*****************************************************************************/

#include <R_ext/PrtUtil.h>
#include <R_ext/Memory.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <float.h>
#include "macros.h"
#include "cosmo.h"
#include "back.h"
#include "fit.h"
#include "matLib.h"
#include "precalc.h"
   

/**********************************************************************/
/*
  getNumRows

  Get the number of rows in a transition matrix of order 'k' given by

  sum_{j=0}^k 4^j

*/
/**********************************************************************/
long getNumRows(
		int order  /* order for which to get number of rows */
		)
{
  int i;             /* index */
  long numRows;      /* number of rows */


  numRows = 1;
  for (i=0; i<order;i++){
    numRows += pow(4,i+1);
  }

  return numRows;
}


/**********************************************************************/
/*
  getTransMat

  Get transition matrix for background Markov model.
  Order may be chosen by likelihood-based cross-validation.
  Memory is allocated for transition matrices and matrices are stored
  in dataset->transMats.
  

*/
/**********************************************************************/
int getTransMat(
		 BOOLEAN cv,           /* find order of transition matrix by CV? */
		 DATASET *dataSet      /* the dataset */
		 )
{
  
  int subOrder;   /* orders from 0 up to the selected order for edge of sequences */ 
  int order;      /* selected order of Markov model */
  long numRows;   /* number of rows in a given transition matrix */

  /* allocate space for transition matrices */
  dataSet->transMats = p2DoubleMalloc(dataSet->maxOrder + 1);
  for (order=0; order<=dataSet->maxOrder; order++){
    numRows = pow(4,order);
    dataSet->transMats[order] = DoubleMalloc(4*numRows);
  }

  
  /* if desired, choose order by cross-validation */
  if (cv) dataSet->order = cvOrder(dataSet);
  if(COSMO_MSG_LEVEL>=3) Rprintf("getTransMat: Estimating transition matrix for order = %d\n\n", dataSet->order);
  
  /* for each order from 0 up to chosen order, estimate the transition matrix */
  for (subOrder=0; subOrder<=dataSet->order; subOrder++){
    if (!estTransMat(subOrder, FALSE, 0, dataSet)){
      Rprintf("getTransMat: Estimated transition matrix of order %d contains zero cells. Try a smaller order. Exiting ...\n", 
	      subOrder);
      return(0);
    }
  }

  if (COSMO_MSG_LEVEL>=4){
    for (subOrder=0; subOrder<=dataSet->order; subOrder++){
      Rprintf("order = %d\n", subOrder);
      PrintDoubleMatrix2File(stderr,dataSet->transMats[subOrder], 4, pow(4,subOrder));
      Rprintf("\n");
    }
  }

  return(1);
}


/**********************************************************************/
/*
  cvOrder  
  
  Choose order of background Markov model by likelihood-based CV.

*/
/**********************************************************************/
int cvOrder(
	    DATASET *dataSet   /* the dataset */
	    )
{
  int order;         /* candidate order */
  int maxOrder;      /* maximum order to consider */
  int subOrder;      /* lower orders */
  int valGroup;      /* validation group */
  int fold;          /* fold for CV */
  double KL;         /* Kullback-Leibler divergence */
  double minKL;      /* minimum Kullback-Leibler divergence */
  BOOLEAN goodLik;   /* is this likelihood valid, i.e. not equal to minus infinity? */
  int bestOrder;     /* best order so far */
  int numSeqs;       /* number of sequences */
  SAMPLE *samples;   /* the samples */
  BOOLEAN goodEst;

  maxOrder = dataSet->maxOrder;
  goodLik  = TRUE;
  goodEst  = TRUE;
  samples  = dataSet->numBackSeqs ? dataSet->backSamples : dataSet->samples;
  numSeqs  = dataSet->numBackSeqs ? dataSet->numBackSeqs : dataSet->numSeqs;
  fold     = dataSet->backFold;

  /* divide samples into validation groups */
  createCvGroups(samples, numSeqs, fold);
  
  bestOrder = 0;
  minKL     = BIG;
  for (order=0; order<maxOrder;order++) dataSet->backKL[order] = 0;
  
  for (order=0; order<=maxOrder && goodEst==TRUE; order++){
    
    KL = 0;
    for (valGroup=0; valGroup<fold; valGroup++){
      
      /* estimate all the transition matrices up to this order from learning set */
      for (subOrder=0; subOrder<=order; subOrder++)goodEst = goodEst && estTransMat(subOrder, TRUE, valGroup, dataSet); 
      
      KL += getMarkovKL(order, &goodLik, TRUE, valGroup, dataSet);
      goodEst = goodEst & goodLik;
      }
    KL /= fold;
    
    KL = goodEst ? KL : BIG;
    dataSet->backKL[order] = KL;
    
    if (COSMO_MSG_LEVEL>=3) Rprintf("order = %d KL = %lf\n", order, KL);
    
    if (KL < minKL){
      minKL = KL;
      bestOrder = order;
    }
  }

  for (; order<=maxOrder; order++) dataSet->backKL[order] = 0;
  
  
  if (COSMO_MSG_LEVEL>=2) Rprintf( "cvOrder: Order of background Markov model estimated as order = %d by CV\n", bestOrder);
  
  return bestOrder;

}


/**********************************************************************/
/*

  getMarkovKl
  
  Get Kullback-Leibler divergence for a given estimate of the transition
  matrix.

  Returns Kullback-Leibler divergence.

*/
/**********************************************************************/
double getMarkovKL(
		   int order,              /* order of transition matrix */
		   BOOLEAN *goodLik,       /* is the likelihood not equal to zero? */
		   BOOLEAN forCV,          /* evaluate Kullback-Leibler divergence on a subset of the sequences? */
		   int valGroup,           /* the validation group on which to evaluate KL divergence */
		   DATASET *dataSet        /* the dataset */
		   )
{

  int seqNum;            /* sequence number */
  int numSeqs;           /* number of sequences */
  int start;             /* position in sequence */
  long seqLength;        /* length of current sequence */
  long numRows;          /* number of rows in transition matrix */
  SAMPLE *sample;        /* current sample */
  SAMPLE *samples;       /* collection of samples */
  double logZero;        /* constant: approximate log(0) */
  double lik;            /* likelihood */ 
  int effNumSeqs;        /* effective number of sequences */
  BOOLEAN back;          /* use background sequences? */

  logZero  = LOG(0);
  samples  = dataSet->numBackSeqs ? dataSet->backSamples : dataSet->samples;
  numSeqs  = dataSet->numBackSeqs ? dataSet->numBackSeqs : dataSet->numSeqs;
  numRows  = pow(4,order);
  *goodLik = TRUE;
  back     = (dataSet->numBackSeqs>0);


  lik = 0;
  effNumSeqs = 0;
  for (seqNum=0; seqNum<numSeqs; seqNum++){
    sample      = samples + seqNum;
    seqLength   = sample->length;

    if (forCV && sample->cvGroup!=valGroup) continue;

    effNumSeqs ++;

    for (start=0; start<seqLength; start++) lik += getLogProbBack(dataSet, seqNum, start, order, back);
  }
    
  return -lik/effNumSeqs;
}


/**********************************************************************/
/*

  estTransMat
  
  Estimate transition matrix for a given order.
  This may be done on a subset of sequences by leaving out those 
  sequences in a specified validation group for CV purposes.

  Returns TRUE if the transition matrix could be estimated without
  zero cells and FALSE otherwise.

*/
/**********************************************************************/
BOOLEAN estTransMat(
		    int order,        /* order of the transition matrix */
		    BOOLEAN forCV,    /* are some subsequences to be left out? */
		    int valGroup,     /* validation group that is to be excluded  */
		    DATASET *dataSet  /* the dataset */
		    )
{
  int seqNum;            /* sequence number */
  int numSeqs;           /* number of sequences */
  int start;             /* position in sequence */
  double *transMat;      /* transition matrix */
  int *seq;              /* current sequence */
  long seqLength;        /* length of current sequence */
  long index;            /* index in transition matrix */
  long numRows;          /* number of rows in transition matrix */
  long rowNum;           /* current row number */
  SAMPLE *sample;        /* current sample */
  SAMPLE *samples;       /* collection of samples */
  int nuc;               /* nucleotide */
  double sum;            /* sum of counts for a given preceding oligo for normalization of counts to frequencies */   
  BOOLEAN goodTransMat;  /* does the transition matrix contain zero cells? */

  
  goodTransMat = TRUE;
  transMat = dataSet->transMats[order];
  samples  = dataSet->numBackSeqs ? dataSet->backSamples : dataSet->samples;
  numSeqs  = dataSet->numBackSeqs ? dataSet->numBackSeqs : dataSet->numSeqs;
  numRows  = pow(4,order);

  /* initialize transition matrix to 0 */
  for (rowNum=0; rowNum < numRows; rowNum++){
    for (nuc=0; nuc<4; nuc++){
      transMat[rowNum*4 + nuc] = 0;
    }
  }

  /* fill transition matrix with observed counts */
  for (seqNum=0; seqNum<numSeqs; seqNum++){
    sample      = samples + seqNum;
    seq         = sample->seq;
    seqLength   = sample->length;


    if (forCV && sample->cvGroup==valGroup) continue;

    for (start=0; start< seqLength-order; start++){

      /* ignore any tuples that contain X */
      if (!containsX(seq+start,order+1,4)){
	index = findIndex(seq+start, order+1);
	transMat[index] ++;
      }
    }
  }

  if (COSMO_MSG_LEVEL>=3 && order<=3) PrintDoubleMatrix2File(stderr,transMat,4,numRows);

  /* convert frequencies to probabilities */
  for (rowNum=0; rowNum<numRows; rowNum++){
    sum = 0;

    for (nuc=0; nuc<4; nuc++) sum += transMat[4*rowNum + nuc];    
    for (nuc=0; nuc<4; nuc++){
      if (transMat[4*rowNum + nuc]==0) goodTransMat = FALSE;
      transMat[4*rowNum + nuc] = LOG(transMat[4*rowNum + nuc]/sum);
    }
  }

  if (COSMO_MSG_LEVEL>=3 && order<=3) PrintDoubleMatrix2File(stderr,transMat,4,numRows);
    
  return goodTransMat;

}

/**********************************************************************/
/*

  containsX
  
  Returns TRUE if the tuple contains the unknown character 'X', coded
  in integer format as the alphabet length. Returns FALSE otherwise.

*/
/**********************************************************************/
BOOLEAN containsX(
		  int * tuple,      /* the tuple to test */
		  int length,       /* length of the tuple */
		  int alen          /* length of the alphabet */
		  )
{
     int i;

     for (i=0; i<length; i++){
       if (tuple[i]>=alen) return(TRUE);
     }

     return(FALSE);      
}
